import { type Extension } from "@codemirror/state";
import { type Command, EditorView } from "@codemirror/view";
import { defaultKeymaps } from "./state.js";
export interface CreateEditOpts {
    prompt: string;
    editorView: EditorView;
    selection: string;
    codeBefore: string;
    codeAfter: string;
    signal?: AbortSignal;
}
export type CompleteFunction = (opts: CreateEditOpts) => Promise<string>;
export interface AiExtensionOptions {
    /** Function to generate completions */
    prompt: CompleteFunction;
    /** Called when user accepts an edit */
    onAcceptEdit?: (opts: CreateEditOpts) => void;
    /** Called when user rejects an edit */
    onRejectEdit?: (opts: CreateEditOpts) => void;
    /** Called when an error occurs during completion */
    onError?: (error: Error) => void;
    /** Custom keymaps */
    keymaps?: Partial<typeof defaultKeymaps>;
    /** Debounce time in ms for input handling */
    inputDebounceTime?: number;
}
/**
 * Creates an AI-assisted editing extension for CodeMirror.
 *
 * @param opts - Configuration options for the AI extension
 * @returns Array of CodeMirror extensions
 * @throws {Error} If required options are missing or invalid
 *
 * Key Features:
 * - AI-assisted code editing with customizable prompts
 * - Keyboard shortcuts for all operations
 * - Undo/redo support
 * - Loading states and error handling
 * - Accessibility support
 * - Input validation and sanitization
 *
 * Usage:
 * ```ts
 * const view = new EditorView({
 *   extensions: [
 *     aiExtension({
 *       prompt: async ({prompt, selection}) => {
 *         // Generate completion
 *         return newCode;
 *       },
 *       onError: (error) => console.error(error)
 *     })
 *   ]
 * });
 * ```
 */
export declare function aiExtension(opts: AiExtensionOptions): Extension[];
export declare const showAiEditInput: Command;
export declare const closeAiEditInput: Command;
export declare const acceptAiEdit: Command;
export declare const rejectAiEdit: Command;
//# sourceMappingURL=inline-edit.d.ts.map